### Persephone - a socio-economic-ecological model of European agricultural landscapes.
###
### This is the top-level file of the Persephone test suite. Execute this to run all tests.
###

using Pkg
Pkg.activate("..")

using Agents
using Dates
using Logging
using LoggingExtras
using Persephone
using Random
using StableRNGs
using Test

const Ps = Persephone

const TESTPARAMETERS = joinpath(pkgdir(Persephone), "test/test_parameters.toml")
const TESTSETTINGS = Ps.getsettings(TESTPARAMETERS)

@testset "Persephone Tests" begin
    @testset "Core model" begin
        include("io_tests.jl")
        include("landscape_tests.jl")
        include("simulation_tests.jl")
    end
    @testset "Nature model" begin
        include("nature_tests.jl")
    end
    @testset "Crop growth model" begin
        include("crop_tests.jl")
    end
    @testset "Farm model" begin
        include("farm_tests.jl")
    end
    rm(TESTSETTINGS["core.outdir"], force=true, recursive=true)
end

# NOTE: Due to an issue with Julia (https://github.com/JuliaLang/julia/issues/48456),
# whenever we are using `@test_logs` with a function that takes a model object, we have
# to wrap that model object in `withtestlogger()`. (For an example, see the "Model
# simulation" testset in simulation_tests.jl.)
